@@ -12,27 +12,29 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Column types that can be configured via the ``column_config`` parameter of ``st.dataframe`` and ``st.data_editor``."""
+"""Column types that can be configured via the ``column_config`` parameter of
+``st.dataframe`` and ``st.data_editor``.
+"""
 
 from __future__ import annotations
 
 __all__ = [
     "AreaChartColumn",
-    "Column",
-    "TextColumn",
-    "NumberColumn",
     "BarChartColumn",
     "CheckboxColumn",
+    "Column",
+    "DateColumn",
     "DatetimeColumn",
     "ImageColumn",
-    "SelectboxColumn",
-    "ProgressColumn",
-    "LinkColumn",
-    "LineChartColumn",
-    "ListColumn",
-    "DateColumn",
-    "TimeColumn",
     "JsonColumn",
+    "LineChartColumn",
+    "LinkColumn",
+    "ListColumn",
+    "NumberColumn",
+    "ProgressColumn",
+    "SelectboxColumn",
+    "TextColumn",
+    "TimeColumn",
 ]
 
 
