@@ -36,8 +36,8 @@ from streamlit.errors import StreamlitAPIException
 # Descriptions of each of the possible config sections.
 # (We use OrderedDict to make the order in which sections are declared in this
 # file be the same order as the sections appear with `streamlit config show`)
-_section_descriptions: dict[str, str] = OrderedDict(
-    _test="Special test section just used for unit tests."
+_section_descriptions: OrderedDict[str, str] = OrderedDict(  # ty: ignore
+    _test="Special test section just used for unit tests."  # ty: ignore
 )
 
 # Ensures that we don't try to get or set config options when config.toml files
@@ -925,6 +925,24 @@ _create_option(
     type_=bool,
 )
 
+_create_option(
+    "server.websocketPingInterval",
+    description="""
+        The interval (in seconds) at which the server pings the client to keep
+        the websocket connection alive.
+
+        The default value should work for most deployments. However, if you're
+        experiencing frequent disconnections in certain proxy setups (e.g.,
+        "Connection error" messages), you may want to try adjusting this value.
+
+        Note: When you set this option, Streamlit automatically sets the ping
+        timeout to match this interval. For Tornado >=6.5, a value less than 30
+        may cause connection issues.
+    """,
+    default_val=None,
+    type_=int,
+)
+
 _create_option(
     "server.enableStaticServing",
     description="""
@@ -1364,13 +1382,13 @@ _create_theme_options(
 
 _create_theme_options(
     "codeFontWeight",
-    categories=["theme"],
+    categories=["theme", CustomThemeCategories.SIDEBAR],
     description="""
         The font weight for code blocks and code text.
 
         This applies to font in inline code, code blocks, `st.json`, and
         `st.help`. This is an integer multiple of 100. Values can be between
-        100 and 900, inclusive.
+        100 and 600, inclusive.
 
         If this isn't set, the code font weight will be 400 (normal weight).
     """,
@@ -1884,7 +1902,7 @@ def get_config_options(
         # Short-circuit if config files were parsed while we were waiting on
         # the lock.
         if _config_options and not force_reparse:
-            return _config_options
+            return _config_options  # ty: ignore[invalid-return-type]
 
         old_options = _config_options
         _config_options = copy.deepcopy(_config_options_template)
