@@ -356,7 +356,13 @@ def prep_chart_data_for_add_rows(
         df.index = pd.RangeIndex(start=start, stop=stop, step=old_step)
         add_rows_metadata.last_index = stop - 1
 
-    out_data, *_ = _prep_data(df, **add_rows_metadata.columns)
+    out_data, *_ = _prep_data(
+        df,
+        x_column=add_rows_metadata.columns["x_column"],
+        y_column_list=add_rows_metadata.columns["y_column_list"],
+        color_column=add_rows_metadata.columns["color_column"],
+        size_column=add_rows_metadata.columns["size_column"],
+    )
 
     return out_data, add_rows_metadata
 
@@ -597,7 +603,7 @@ def _maybe_reset_index_in_place(
             x_column = _SEPARATED_INDEX_COLUMN_NAME
         else:
             # Reuse index's name for the new column.
-            x_column = df.index.name
+            x_column = str(df.index.name)
 
         df.index.name = x_column
         df.reset_index(inplace=True)  # noqa: PD002
@@ -988,7 +994,9 @@ def _get_color_encoding(
                 return alt.ColorValue(to_css_color(cast("Any", color_value[0])))
             return alt.Color(
                 field=color_column if color_column is not None else alt.Undefined,
-                scale=alt.Scale(range=[to_css_color(c) for c in color_values]),
+                scale=alt.Scale(
+                    domain=y_column_list, range=[to_css_color(c) for c in color_values]
+                ),
                 legend=_COLOR_LEGEND_SETTINGS,
                 type="nominal",
                 title=" ",
