@@ -31,6 +31,11 @@ from typing import (
 from typing_extensions import TypeAlias
 
 from streamlit.elements.lib.form_utils import current_form_id
+from streamlit.elements.lib.layout_utils import (
+    LayoutConfig,
+    Width,
+    validate_width,
+)
 from streamlit.elements.lib.options_selector_utils import (
     check_and_convert_to_indices,
     convert_to_sequence_and_check_comparable,
@@ -54,7 +59,6 @@ from streamlit.runtime.metrics_util import gather_metrics
 from streamlit.runtime.scriptrunner_utils.script_run_context import get_script_run_ctx
 from streamlit.runtime.state import register_widget
 from streamlit.string_util import is_emoji, validate_material_icon
-from streamlit.type_util import T
 
 if TYPE_CHECKING:
     from streamlit.dataframe_util import OptionSequence
@@ -70,7 +74,7 @@ if TYPE_CHECKING:
         WidgetSerializer,
     )
 
-
+T = TypeVar("T")
 V = TypeVar("V")
 
 _THUMB_ICONS: Final = (":material/thumb_up:", ":material/thumb_down:")
@@ -107,11 +111,7 @@ class _MultiSelectSerde(Generic[T]):
         indices = check_and_convert_to_indices(self.options, value)
         return indices if indices is not None else []
 
-    def deserialize(
-        self,
-        ui_value: list[int] | None,
-        widget_id: str = "",
-    ) -> list[T]:
+    def deserialize(self, ui_value: list[int] | None) -> list[T]:
         current_value: list[int] = (
             ui_value if ui_value is not None else self.default_value
         )
@@ -145,8 +145,8 @@ class _SingleSelectSerde(Generic[T]):
         _value = [value] if value is not None else []
         return self.multiselect_serde.serialize(_value)
 
-    def deserialize(self, ui_value: list[int] | None, widget_id: str = "") -> T | None:
-        deserialized = self.multiselect_serde.deserialize(ui_value, widget_id)
+    def deserialize(self, ui_value: list[int] | None) -> T | None:
+        deserialized = self.multiselect_serde.deserialize(ui_value)
 
         if len(deserialized) == 0:
             return None
@@ -169,7 +169,7 @@ class ButtonGroupSerde(Generic[T]):
         options: Sequence[T],
         default_values: list[int],
         type: Literal["single", "multi"],
-    ):
+    ) -> None:
         self.options = options
         self.default_values = default_values
         self.type = type
@@ -182,10 +182,8 @@ class ButtonGroupSerde(Generic[T]):
     def serialize(self, value: T | list[T] | None) -> list[int]:
         return self.serde.serialize(cast("Any", value))
 
-    def deserialize(
-        self, ui_value: list[int] | None, widget_id: str = ""
-    ) -> list[T] | T | None:
-        return self.serde.deserialize(ui_value, widget_id)
+    def deserialize(self, ui_value: list[int] | None) -> list[T] | T | None:
+        return self.serde.deserialize(ui_value)
 
 
 def get_mapped_options(
@@ -256,7 +254,7 @@ def _build_proto(
     return proto
 
 
-def _maybe_raise_selection_mode_warning(selection_mode: SelectionMode):
+def _maybe_raise_selection_mode_warning(selection_mode: SelectionMode) -> None:
     """Check if the selection_mode value is valid or raise exception otherwise."""
     if selection_mode not in ["single", "multi"]:
         raise StreamlitAPIException(
@@ -282,6 +280,7 @@ class ButtonGroupMixin:
         on_change: WidgetCallback | None = None,
         args: WidgetArgs | None = None,
         kwargs: WidgetKwargs | None = None,
+        width: Width = "content",
     ) -> Literal[0, 1] | None: ...
     @overload
     def feedback(
@@ -293,6 +292,7 @@ class ButtonGroupMixin:
         on_change: WidgetCallback | None = None,
         args: WidgetArgs | None = None,
         kwargs: WidgetKwargs | None = None,
+        width: Width = "content",
     ) -> Literal[0, 1, 2, 3, 4] | None: ...
     @gather_metrics("feedback")
     def feedback(
@@ -304,6 +304,7 @@ class ButtonGroupMixin:
         on_change: WidgetCallback | None = None,
         args: WidgetArgs | None = None,
         kwargs: WidgetKwargs | None = None,
+        width: Width = "content",
     ) -> int | None:
         """Display a feedback widget.
 
@@ -313,7 +314,7 @@ class ButtonGroupMixin:
 
         Parameters
         ----------
-        options: "thumbs", "faces", or "stars"
+        options : "thumbs", "faces", or "stars"
             The feedback options displayed to the user. ``options`` can be one
             of the following:
 
@@ -338,12 +339,25 @@ class ButtonGroupMixin:
             An optional callback invoked when this feedback widget's value
             changes.
 
-        args : tuple
-            An optional tuple of args to pass to the callback.
+        args : list or tuple
+            An optional list or tuple of args to pass to the callback.
 
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
 
+        width : "content", "stretch", or int
+            The width of the feedback widget. This can be one of the following:
+
+            - ``"content"`` (default): The width of the widget matches the
+              width of its content, but doesn't exceed the width of the parent
+              container.
+            - ``"stretch"``: The width of the widget matches the width of the
+              parent container.
+            - An integer specifying the width in pixels: The widget has a
+              fixed width. If the specified width is greater than the width of
+              the parent container, the width of the widget matches the width
+              of the parent container.
+
         Returns
         -------
         int or None
@@ -414,6 +428,7 @@ class ButtonGroupMixin:
             kwargs=kwargs,
             selection_visualization=selection_visualization,
             style="borderless",
+            width=width,
         )
         return sentiment.value
 
@@ -433,6 +448,7 @@ class ButtonGroupMixin:
         kwargs: WidgetKwargs | None = None,
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: Width = "content",
     ) -> V | None: ...
     @overload
     def pills(
@@ -450,6 +466,7 @@ class ButtonGroupMixin:
         kwargs: WidgetKwargs | None = None,
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: Width = "content",
     ) -> list[V]: ...
     @gather_metrics("pills")
     def pills(
@@ -467,6 +484,7 @@ class ButtonGroupMixin:
         kwargs: WidgetKwargs | None = None,
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: Width = "content",
     ) -> list[V] | V | None:
         r"""Display a pills widget.
 
@@ -476,7 +494,7 @@ class ButtonGroupMixin:
 
         Parameters
         ----------
-        label: str
+        label : str
             A short label explaining to the user what this widget is for.
             The label can optionally contain GitHub-flavored Markdown of the
             following types: Bold, Italics, Strikethroughs, Inline Code, Links,
@@ -498,7 +516,7 @@ class ButtonGroupMixin:
             .. |st.markdown| replace:: ``st.markdown``
             .. _st.markdown: https://docs.streamlit.io/develop/api-reference/text/st.markdown
 
-        options: Iterable of V
+        options : Iterable of V
             Labels for the select options in an ``Iterable``. This can be a
             ``list``, ``set``, or anything supported by ``st.dataframe``. If
             ``options`` is dataframe-like, the first column will be used. Each
@@ -506,18 +524,18 @@ class ButtonGroupMixin:
             optionally contain GitHub-flavored Markdown, including the Markdown
             directives described in the ``body`` parameter of ``st.markdown``.
 
-        selection_mode: "single" or "multi"
+        selection_mode : "single" or "multi"
             The selection mode for the widget. If this is ``"single"``
             (default), only one option can be selected. If this is ``"multi"``,
             multiple options can be selected.
 
-        default: Iterable of V, V, or None
+        default : Iterable of V, V, or None
             The value of the widget when it first renders. If the
             ``selection_mode`` is ``multi``, this can be a list of values, a
             single value, or ``None``. If the ``selection_mode`` is
             ``"single"``, this can be a single value or ``None``.
 
-        format_func: function
+        format_func : function
             Function to modify the display of the options. It receives
             the raw option as an argument and should output the label to be
             shown for that option. This has no impact on the return value of
@@ -525,13 +543,13 @@ class ButtonGroupMixin:
             Markdown, including the Markdown directives described in the
             ``body`` parameter of ``st.markdown``.
 
-        key: str or int
+        key : str or int
             An optional string or integer to use as the unique key for the widget.
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
 
-        help: str or None
+        help : str or None
             A tooltip that gets displayed next to the widget label. Streamlit
             only displays the tooltip when ``label_visibility="visible"``. If
             this is ``None`` (default), no tooltip is displayed.
@@ -540,25 +558,38 @@ class ButtonGroupMixin:
             including the Markdown directives described in the ``body``
             parameter of ``st.markdown``.
 
-        on_change: callable
+        on_change : callable
             An optional callback invoked when this widget's value changes.
 
-        args: tuple
-            An optional tuple of args to pass to the callback.
+        args : list or tuple
+            An optional list or tuple of args to pass to the callback.
 
-        kwargs: dict
+        kwargs : dict
             An optional dict of kwargs to pass to the callback.
 
-        disabled: bool
+        disabled : bool
             An optional boolean that disables the widget if set to ``True``.
             The default is ``False``.
 
-        label_visibility: "visible", "hidden", or "collapsed"
+        label_visibility : "visible", "hidden", or "collapsed"
             The visibility of the label. The default is ``"visible"``. If this
             is ``"hidden"``, Streamlit displays an empty spacer instead of the
-            label, which can help keep the widget alligned with other widgets.
+            label, which can help keep the widget aligned with other widgets.
             If this is ``"collapsed"``, Streamlit displays no label or spacer.
 
+        width : "content", "stretch", or int
+            The width of the pills widget. This can be one of the following:
+
+            - ``"content"`` (default): The width of the widget matches the
+              width of its content, but doesn't exceed the width of the parent
+              container.
+            - ``"stretch"``: The width of the widget matches the width of the
+              parent container.
+            - An integer specifying the width in pixels: The widget has a
+              fixed width. If the specified width is greater than the width of
+              the parent container, the width of the widget matches the width
+              of the parent container.
+
         Returns
         -------
         list of V, V, or None
@@ -624,6 +655,7 @@ class ButtonGroupMixin:
             kwargs=kwargs,
             disabled=disabled,
             label_visibility=label_visibility,
+            width=width,
         )
 
     @overload
@@ -642,6 +674,7 @@ class ButtonGroupMixin:
         kwargs: WidgetKwargs | None = None,
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: Width = "content",
     ) -> V | None: ...
     @overload
     def segmented_control(
@@ -659,6 +692,7 @@ class ButtonGroupMixin:
         kwargs: WidgetKwargs | None = None,
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: Width = "content",
     ) -> list[V]: ...
 
     @gather_metrics("segmented_control")
@@ -677,6 +711,7 @@ class ButtonGroupMixin:
         kwargs: WidgetKwargs | None = None,
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: Width = "content",
     ) -> list[V] | V | None:
         r"""Display a segmented control widget.
 
@@ -707,7 +742,7 @@ class ButtonGroupMixin:
             .. |st.markdown| replace:: ``st.markdown``
             .. _st.markdown: https://docs.streamlit.io/develop/api-reference/text/st.markdown
 
-        options: Iterable of V
+        options : Iterable of V
             Labels for the select options in an ``Iterable``. This can be a
             ``list``, ``set``, or anything supported by ``st.dataframe``. If
             ``options`` is dataframe-like, the first column will be used. Each
@@ -715,18 +750,18 @@ class ButtonGroupMixin:
             optionally contain GitHub-flavored Markdown, including the Markdown
             directives described in the ``body`` parameter of ``st.markdown``.
 
-        selection_mode: "single" or "multi"
+        selection_mode : "single" or "multi"
             The selection mode for the widget. If this is ``"single"``
             (default), only one option can be selected. If this is ``"multi"``,
             multiple options can be selected.
 
-        default: Iterable of V, V, or None
+        default : Iterable of V, V, or None
             The value of the widget when it first renders. If the
             ``selection_mode`` is ``multi``, this can be a list of values, a
             single value, or ``None``. If the ``selection_mode`` is
             ``"single"``, this can be a single value or ``None``.
 
-        format_func: function
+        format_func : function
             Function to modify the display of the options. It receives
             the raw option as an argument and should output the label to be
             shown for that option. This has no impact on the return value of
@@ -734,13 +769,13 @@ class ButtonGroupMixin:
             Markdown, including the Markdown directives described in the
             ``body`` parameter of ``st.markdown``.
 
-        key: str or int
+        key : str or int
             An optional string or integer to use as the unique key for the widget.
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
 
-        help: str or None
+        help : str or None
             A tooltip that gets displayed next to the widget label. Streamlit
             only displays the tooltip when ``label_visibility="visible"``. If
             this is ``None`` (default), no tooltip is displayed.
@@ -749,25 +784,39 @@ class ButtonGroupMixin:
             including the Markdown directives described in the ``body``
             parameter of ``st.markdown``.
 
-        on_change: callable
+        on_change : callable
             An optional callback invoked when this widget's value changes.
 
-        args: tuple
-            An optional tuple of args to pass to the callback.
+        args : list or tuple
+            An optional list or tuple of args to pass to the callback.
 
-        kwargs: dict
+        kwargs : dict
             An optional dict of kwargs to pass to the callback.
 
-        disabled: bool
+        disabled : bool
             An optional boolean that disables the widget if set to ``True``.
             The default is ``False``.
 
-        label_visibility: "visible", "hidden", or "collapsed"
+        label_visibility : "visible", "hidden", or "collapsed"
             The visibility of the label. The default is ``"visible"``. If this
             is ``"hidden"``, Streamlit displays an empty spacer instead of the
-            label, which can help keep the widget alligned with other widgets.
+            label, which can help keep the widget aligned with other widgets.
             If this is ``"collapsed"``, Streamlit displays no label or spacer.
 
+        width : "content", "stretch", or int
+            The width of the segmented control widget. This can be one of the
+            following:
+
+            - ``"content"`` (default): The width of the widget matches the
+              width of its content, but doesn't exceed the width of the parent
+              container.
+            - ``"stretch"``: The width of the widget matches the width of the
+              parent container.
+            - An integer specifying the width in pixels: The widget has a
+              fixed width. If the specified width is greater than the width of
+              the parent container, the width of the widget matches the width
+              of the parent container.
+
         Returns
         -------
         list of V, V, or None
@@ -836,6 +885,7 @@ class ButtonGroupMixin:
             kwargs=kwargs,
             disabled=disabled,
             label_visibility=label_visibility,
+            width=width,
         )
 
     @gather_metrics("_internal_button_group")
@@ -855,6 +905,7 @@ class ButtonGroupMixin:
         label: str | None = None,
         label_visibility: LabelVisibility = "visible",
         help: str | None = None,
+        width: Width = "content",
     ) -> list[V] | V | None:
         maybe_raise_label_warnings(label, label_visibility)
 
@@ -908,6 +959,7 @@ class ButtonGroupMixin:
             kwargs=kwargs,
             label=label,
             label_visibility=label_visibility,
+            width=width,
         )
 
         if selection_mode == "multi":
@@ -938,6 +990,7 @@ class ButtonGroupMixin:
         label: str | None = None,
         label_visibility: LabelVisibility = "visible",
         help: str | None = None,
+        width: Width = "content",
     ) -> RegisterWidgetResult[T]:
         _maybe_raise_selection_mode_warning(selection_mode)
 
@@ -973,6 +1026,9 @@ class ButtonGroupMixin:
         if default is not None and len(default) == 0:
             _default = None
 
+        validate_width(width, allow_content=True)
+        layout_config = LayoutConfig(width=width)
+
         check_widget_policies(self.dg, key, on_change, default_value=_default)
 
         widget_name = "button_group"
@@ -990,10 +1046,14 @@ class ButtonGroupMixin:
             widget_name,
             user_key=key,
             form_id=form_id,
+            dg=self.dg,
             options=formatted_options,
             default=default,
             click_mode=parsed_selection_mode,
             style=style,
+            width=width,
+            label=label,
+            help=help,
         )
 
         proto = _build_proto(
@@ -1028,7 +1088,7 @@ class ButtonGroupMixin:
         if ctx:
             save_for_app_testing(ctx, element_id, format_func)
 
-        self.dg._enqueue(widget_name, proto)
+        self.dg._enqueue(widget_name, proto, layout_config=layout_config)
 
         return widget_state
 
