@@ -31,6 +31,11 @@ from typing import (
 from typing_extensions import TypeAlias
 
 from streamlit.elements.lib.form_utils import current_form_id
+from streamlit.elements.lib.layout_utils import (
+    LayoutConfig,
+    WidthWithoutContent,
+    validate_width,
+)
 from streamlit.elements.lib.policies import (
     check_widget_policies,
     maybe_raise_label_warnings,
@@ -263,7 +268,7 @@ class _DateInputValues:
 class TimeInputSerde:
     value: time | None
 
-    def deserialize(self, ui_value: str | None, widget_id: Any = "") -> time | None:
+    def deserialize(self, ui_value: str | None) -> time | None:
         return (
             datetime.strptime(ui_value, "%H:%M").time()
             if ui_value is not None
@@ -282,11 +287,7 @@ class TimeInputSerde:
 class DateInputSerde:
     value: _DateInputValues
 
-    def deserialize(
-        self,
-        ui_value: Any,
-        widget_id: str = "",
-    ) -> DateWidgetReturn:
+    def deserialize(self, ui_value: Any) -> DateWidgetReturn:
         return_value: Sequence[date] | None
         if ui_value is not None:
             return_value = tuple(
@@ -325,6 +326,7 @@ class TimeWidgetsMixin:
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
         step: int | timedelta = timedelta(minutes=DEFAULT_STEP_MINUTES),
+        width: WidthWithoutContent = "stretch",
     ) -> time:
         pass
 
@@ -342,6 +344,7 @@ class TimeWidgetsMixin:
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
         step: int | timedelta = timedelta(minutes=DEFAULT_STEP_MINUTES),
+        width: WidthWithoutContent = "stretch",
     ) -> time | None:
         pass
 
@@ -359,6 +362,7 @@ class TimeWidgetsMixin:
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
         step: int | timedelta = timedelta(minutes=DEFAULT_STEP_MINUTES),
+        width: WidthWithoutContent = "stretch",
     ) -> time | None:
         r"""Display a time input widget.
 
@@ -416,8 +420,8 @@ class TimeWidgetsMixin:
         on_change : callable
             An optional callback invoked when this time_input's value changes.
 
-        args : tuple
-            An optional tuple of args to pass to the callback.
+        args : list or tuple
+            An optional list or tuple of args to pass to the callback.
 
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
@@ -429,13 +433,23 @@ class TimeWidgetsMixin:
         label_visibility : "visible", "hidden", or "collapsed"
             The visibility of the label. The default is ``"visible"``. If this
             is ``"hidden"``, Streamlit displays an empty spacer instead of the
-            label, which can help keep the widget alligned with other widgets.
+            label, which can help keep the widget aligned with other widgets.
             If this is ``"collapsed"``, Streamlit displays no label or spacer.
 
         step : int or timedelta
             The stepping interval in seconds. Defaults to 900, i.e. 15 minutes.
             You can also pass a datetime.timedelta object.
 
+        width : "stretch" or int
+            The width of the time input widget. This can be one of the following:
+
+            - ``"stretch"`` (default): The width of the widget matches the
+              width of the parent container.
+            - An integer specifying the width in pixels: The widget has a
+              fixed width. If the specified width is greater than the width of
+              the parent container, the width of the widget matches the width
+              of the parent container.
+
         Returns
         -------
         datetime.time or None
@@ -479,6 +493,7 @@ class TimeWidgetsMixin:
             disabled=disabled,
             label_visibility=label_visibility,
             step=step,
+            width=width,
             ctx=ctx,
         )
 
@@ -495,6 +510,7 @@ class TimeWidgetsMixin:
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
         step: int | timedelta = timedelta(minutes=DEFAULT_STEP_MINUTES),
+        width: WidthWithoutContent = "stretch",
         ctx: ScriptRunContext | None = None,
     ) -> time | None:
         key = to_key(key)
@@ -508,19 +524,18 @@ class TimeWidgetsMixin:
         maybe_raise_label_warnings(label, label_visibility)
 
         parsed_time: time | None
-        if value is None:
-            parsed_time = None
-        else:
-            parsed_time = _convert_timelike_to_time(value)
+        parsed_time = None if value is None else _convert_timelike_to_time(value)
 
         element_id = compute_and_register_element_id(
             "time_input",
             user_key=key,
             form_id=current_form_id(self.dg),
+            dg=self.dg,
             label=label,
             value=parsed_time if isinstance(value, (datetime, time)) else value,
             help=help,
             step=step,
+            width=width,
         )
         del value
 
@@ -570,7 +585,10 @@ class TimeWidgetsMixin:
                 time_input_proto.value = serialized_value
             time_input_proto.set_value = True
 
-        self.dg._enqueue("time_input", time_input_proto)
+        validate_width(width)
+        layout_config = LayoutConfig(width=width)
+
+        self.dg._enqueue("time_input", time_input_proto, layout_config=layout_config)
         return widget_state.value
 
     @overload
@@ -589,6 +607,7 @@ class TimeWidgetsMixin:
         format: str = "YYYY/MM/DD",
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: WidthWithoutContent = "stretch",
     ) -> date: ...
 
     @overload
@@ -607,6 +626,7 @@ class TimeWidgetsMixin:
         format: str = "YYYY/MM/DD",
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: WidthWithoutContent = "stretch",
     ) -> date | None: ...
 
     @overload
@@ -627,6 +647,7 @@ class TimeWidgetsMixin:
         format: str = "YYYY/MM/DD",
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: WidthWithoutContent = "stretch",
     ) -> DateWidgetRangeReturn: ...
 
     @gather_metrics("date_input")
@@ -645,6 +666,7 @@ class TimeWidgetsMixin:
         format: str = "YYYY/MM/DD",
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: WidthWithoutContent = "stretch",
     ) -> DateWidgetReturn:
         r"""Display a date input widget.
 
@@ -731,8 +753,8 @@ class TimeWidgetsMixin:
         on_change : callable
             An optional callback invoked when this date_input's value changes.
 
-        args : tuple
-            An optional tuple of args to pass to the callback.
+        args : list or tuple
+            An optional list or tuple of args to pass to the callback.
 
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
@@ -749,9 +771,19 @@ class TimeWidgetsMixin:
         label_visibility : "visible", "hidden", or "collapsed"
             The visibility of the label. The default is ``"visible"``. If this
             is ``"hidden"``, Streamlit displays an empty spacer instead of the
-            label, which can help keep the widget alligned with other widgets.
+            label, which can help keep the widget aligned with other widgets.
             If this is ``"collapsed"``, Streamlit displays no label or spacer.
 
+        width : "stretch" or int
+            The width of the date input widget. This can be one of the following:
+
+            - ``"stretch"`` (default): The width of the widget matches the
+              width of the parent container.
+            - An integer specifying the width in pixels: The widget has a
+              fixed width. If the specified width is greater than the width of
+              the parent container, the width of the widget matches the width
+              of the parent container.
+
         Returns
         -------
         datetime.date or a tuple with 0-2 dates or None
@@ -818,6 +850,7 @@ class TimeWidgetsMixin:
             disabled=disabled,
             label_visibility=label_visibility,
             format=format,
+            width=width,
             ctx=ctx,
         )
 
@@ -836,6 +869,7 @@ class TimeWidgetsMixin:
         format: str = "YYYY/MM/DD",
         disabled: bool = False,
         label_visibility: LabelVisibility = "visible",
+        width: WidthWithoutContent = "stretch",
         ctx: ScriptRunContext | None = None,
     ) -> DateWidgetReturn:
         key = to_key(key)
@@ -869,10 +903,7 @@ class TimeWidgetsMixin:
         if value == "today":
             parsed = None
         elif isinstance(value, Sequence):
-            parsed = [
-                parse_date_deterministic_for_id(cast("NullableScalarDateValue", v))
-                for v in value
-            ]
+            parsed = [parse_date_deterministic_for_id(v) for v in value]
         else:
             parsed = parse_date_deterministic_for_id(value)
 
@@ -882,12 +913,14 @@ class TimeWidgetsMixin:
             "date_input",
             user_key=key,
             form_id=current_form_id(self.dg),
+            dg=self.dg,
             label=label,
             value=parsed,
             min_value=parsed_min_date,
             max_value=parsed_max_date,
             help=help,
             format=format,
+            width=width,
         )
         if not bool(ALLOWED_DATE_FORMATS.match(format)):
             raise StreamlitAPIException(
@@ -962,7 +995,10 @@ class TimeWidgetsMixin:
             date_input_proto.value[:] = serde.serialize(widget_state.value)
             date_input_proto.set_value = True
 
-        self.dg._enqueue("date_input", date_input_proto)
+        validate_width(width)
+        layout_config = LayoutConfig(width=width)
+
+        self.dg._enqueue("date_input", date_input_proto, layout_config=layout_config)
         return widget_state.value
 
     @property
