@@ -12,16 +12,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+
 """@st.cache_resource implementation."""
 
 from __future__ import annotations
 
 import math
 import threading
-from typing import TYPE_CHECKING, Any, Callable, Final, TypeVar, cast, overload
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Final,
+    TypeVar,
+    overload,
+)
 
 from cachetools import TTLCache
-from typing_extensions import TypeAlias
+from typing_extensions import ParamSpec, TypeAlias
 
 import streamlit as st
 from streamlit.logger import get_logger
@@ -30,6 +38,7 @@ from streamlit.runtime.caching.cache_errors import CacheKeyNotFoundError
 from streamlit.runtime.caching.cache_type import CacheType
 from streamlit.runtime.caching.cache_utils import (
     Cache,
+    CachedFunc,
     CachedFuncInfo,
     make_cached_func_wrapper,
 )
@@ -44,7 +53,6 @@ from streamlit.runtime.stats import CacheStat, CacheStatsProvider, group_stats
 from streamlit.time_util import time_to_seconds
 
 if TYPE_CHECKING:
-    import types
     from datetime import timedelta
 
     from streamlit.runtime.caching.hashing import HashFuncsDict
@@ -69,9 +77,9 @@ def _equal_validate_funcs(a: ValidateFunc | None, b: ValidateFunc | None) -> boo
 class ResourceCaches(CacheStatsProvider):
     """Manages all ResourceCache instances."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         self._caches_lock = threading.Lock()
-        self._function_caches: dict[str, ResourceCache] = {}
+        self._function_caches: dict[str, ResourceCache[Any]] = {}
 
     def get_cache(
         self,
@@ -80,7 +88,7 @@ class ResourceCaches(CacheStatsProvider):
         max_entries: int | float | None,
         ttl: float | timedelta | str | None,
         validate: ValidateFunc | None,
-    ) -> ResourceCache:
+    ) -> ResourceCache[Any]:
         """Return the mem cache for the given key.
 
         If it doesn't exist, create a new one with the given params.
@@ -140,22 +148,28 @@ def get_resource_cache_stats_provider() -> CacheStatsProvider:
     return _resource_caches
 
 
-class CachedResourceFuncInfo(CachedFuncInfo):
+P = ParamSpec("P")
+R = TypeVar("R")
+
+
+class CachedResourceFuncInfo(CachedFuncInfo[P, R]):
     """Implements the CachedFuncInfo interface for @st.cache_resource."""
 
     def __init__(
         self,
-        func: types.FunctionType,
+        func: Callable[P, R],
         show_spinner: bool | str,
         max_entries: int | None,
         ttl: float | timedelta | str | None,
         validate: ValidateFunc | None,
         hash_funcs: HashFuncsDict | None = None,
-    ):
+        show_time: bool = False,
+    ) -> None:
         super().__init__(
             func,
-            show_spinner=show_spinner,
             hash_funcs=hash_funcs,
+            show_spinner=show_spinner,
+            show_time=show_time,
         )
         self.max_entries = max_entries
         self.ttl = ttl
@@ -174,7 +188,7 @@ class CachedResourceFuncInfo(CachedFuncInfo):
         """A human-readable name for the cached function."""
         return f"{self.func.__module__}.{self.func.__qualname__}"
 
-    def get_function_cache(self, function_key: str) -> Cache:
+    def get_function_cache(self, function_key: str) -> Cache[R]:
         return _resource_caches.get_cache(
             key=function_key,
             display_name=self.display_name,
@@ -189,7 +203,7 @@ class CacheResourceAPI:
     and st.cache_resource.clear().
     """
 
-    def __init__(self, decorator_metric_name: str):
+    def __init__(self, decorator_metric_name: str) -> None:
         """Create a CacheResourceAPI instance.
 
         Parameters
@@ -205,11 +219,9 @@ class CacheResourceAPI:
     # Type-annotate the decorator function.
     # (See https://mypy.readthedocs.io/en/stable/generics.html#decorator-factories)
 
-    F = TypeVar("F", bound=Callable[..., Any])
-
     # Bare decorator usage
     @overload
-    def __call__(self, func: F) -> F: ...
+    def __call__(self, func: Callable[P, R]) -> CachedFunc[P, R]: ...
 
     # Decorator with arguments
     @overload
@@ -219,27 +231,30 @@ class CacheResourceAPI:
         ttl: float | timedelta | str | None = None,
         max_entries: int | None = None,
         show_spinner: bool | str = True,
+        show_time: bool = False,
         validate: ValidateFunc | None = None,
         experimental_allow_widgets: bool = False,
         hash_funcs: HashFuncsDict | None = None,
-    ) -> Callable[[F], F]: ...
+    ) -> Callable[[Callable[P, R]], CachedFunc[P, R]]: ...
 
     def __call__(
         self,
-        func: F | None = None,
+        func: Callable[P, R] | None = None,
         *,
         ttl: float | timedelta | str | None = None,
         max_entries: int | None = None,
         show_spinner: bool | str = True,
+        show_time: bool = False,
         validate: ValidateFunc | None = None,
         experimental_allow_widgets: bool = False,
         hash_funcs: HashFuncsDict | None = None,
-    ):
+    ) -> CachedFunc[P, R] | Callable[[Callable[P, R]], CachedFunc[P, R]]:
         return self._decorator(
             func,
             ttl=ttl,
             max_entries=max_entries,
             show_spinner=show_spinner,
+            show_time=show_time,
             validate=validate,
             experimental_allow_widgets=experimental_allow_widgets,
             hash_funcs=hash_funcs,
@@ -247,15 +262,16 @@ class CacheResourceAPI:
 
     def _decorator(
         self,
-        func: F | None,
+        func: Callable[P, R] | None,
         *,
         ttl: float | timedelta | str | None,
         max_entries: int | None,
         show_spinner: bool | str,
+        show_time: bool = False,
         validate: ValidateFunc | None,
         experimental_allow_widgets: bool,
         hash_funcs: HashFuncsDict | None = None,
-    ):
+    ) -> CachedFunc[P, R] | Callable[[Callable[P, R]], CachedFunc[P, R]]:
         """Decorator to cache functions that return global resources (e.g. database connections, ML models).
 
         Cached objects are shared across all users, sessions, and reruns. They
@@ -273,8 +289,29 @@ class CacheResourceAPI:
         arguments match a previous function call. Alternatively, you can
         declare custom hashing functions with ``hash_funcs``.
 
-        To cache data, use ``st.cache_data`` instead. Learn more about caching at
-        https://docs.streamlit.io/develop/concepts/architecture/caching.
+        Cached values are available to all users of your app. If you need to
+        save results that should only be accessible within a session, use
+        `Session State
+        <https://docs.streamlit.io/develop/concepts/architecture/session-state>`_
+        instead. Within each user session, an ``@st.cache_resource``-decorated
+        function returns the cached instance of the return value (if the value
+        is already cached). Therefore, objects cached by ``st.cache_resource``
+        act like singletons and can mutate. To cache data and return copies,
+        use ``st.cache_data`` instead. To learn more about caching, see
+        `Caching overview
+        <https://docs.streamlit.io/develop/concepts/architecture/caching>`_.
+
+        .. warning::
+            Async objects are not officially supported in Streamlit. Caching
+            async objects or objects that reference async objects may have
+            unintended consequences. For example, Streamlit may close event
+            loops in its normal operation and make the cached object raise an
+            ``Event loop closed`` error.
+
+            To upvote official ``asyncio`` support, see GitHub issue `#8488
+            <https://github.com/streamlit/streamlit/issues/8488>`_. To upvote
+            support for caching async functions, see GitHub issue `#8308
+            <https://github.com/streamlit/streamlit/issues/8308>`_.
 
         Parameters
         ----------
@@ -304,6 +341,12 @@ class CacheResourceAPI:
             a "cache miss" and the cached resource is being created. If string,
             value of show_spinner param will be used for spinner text.
 
+        show_time : bool
+            Whether to show the elapsed time next to the spinner text. If this is
+            ``False`` (default), no time is displayed. If this is ``True``,
+            elapsed time is displayed with a precision of 0.1 seconds. The time
+            format is not configurable.
+
         validate : callable or None
             An optional validation function for cached data. ``validate`` is called
             each time the cached value is accessed. It receives the cached value as
@@ -421,6 +464,7 @@ class CacheResourceAPI:
                 CachedResourceFuncInfo(
                     func=f,
                     show_spinner=show_spinner,
+                    show_time=show_time,
                     max_entries=max_entries,
                     ttl=ttl,
                     validate=validate,
@@ -430,8 +474,9 @@ class CacheResourceAPI:
 
         return make_cached_func_wrapper(
             CachedResourceFuncInfo(
-                func=cast("types.FunctionType", func),
+                func=func,
                 show_spinner=show_spinner,
+                show_time=show_time,
                 max_entries=max_entries,
                 ttl=ttl,
                 validate=validate,
@@ -445,7 +490,7 @@ class CacheResourceAPI:
         _resource_caches.clear_all()
 
 
-class ResourceCache(Cache):
+class ResourceCache(Cache[R]):
     """Manages cached values for a single st.cache_resource function."""
 
     def __init__(
@@ -455,11 +500,11 @@ class ResourceCache(Cache):
         ttl_seconds: float,
         validate: ValidateFunc | None,
         display_name: str,
-    ):
+    ) -> None:
         super().__init__()
         self.key = key
         self.display_name = display_name
-        self._mem_cache: TTLCache[str, CachedResult] = TTLCache(
+        self._mem_cache: TTLCache[str, CachedResult[R]] = TTLCache(
             maxsize=max_entries, ttl=ttl_seconds, timer=cache_utils.TTLCACHE_TIMER
         )
         self._mem_cache_lock = threading.Lock()
@@ -473,7 +518,7 @@ class ResourceCache(Cache):
     def ttl_seconds(self) -> float:
         return self._mem_cache.ttl
 
-    def read_result(self, key: str) -> CachedResult:
+    def read_result(self, key: str) -> CachedResult[R]:
         """Read a value and associated messages from the cache.
         Raise `CacheKeyNotFoundError` if the value doesn't exist.
         """
@@ -492,7 +537,7 @@ class ResourceCache(Cache):
             return result
 
     @gather_metrics("_cache_resource_object")
-    def write_result(self, key: str, value: Any, messages: list[MsgData]) -> None:
+    def write_result(self, key: str, value: R, messages: list[MsgData]) -> None:
         """Write a value and associated messages to the cache."""
         main_id = st._main.id
         sidebar_id = st.sidebar.id
