@@ -23,11 +23,14 @@ from __future__ import annotations
 import os
 import time
 from pathlib import Path
-from typing import Callable, TypeVar
+from typing import TYPE_CHECKING, Callable, TypeVar
 
-from streamlit.errors import Error
+from streamlit.errors import StreamlitMaxRetriesError
 from streamlit.util import calc_md5
 
+if TYPE_CHECKING:
+    from collections.abc import Generator
+
 # How many times to try to grab the MD5 hash.
 _MAX_RETRIES = 5
 
@@ -167,20 +170,21 @@ def _do_with_retries(
             file_path, # For pretty error message.
         )
     """
+
     for i in _retry_dance():
         try:
             return orig_fn()
-        except exceptions:
+        except exceptions as ex:  # noqa: PERF203
             if i >= _MAX_RETRIES - 1:
-                raise
-            else:
-                # Continue with loop to either retry or raise MaxRetriesError.
-                pass
+                raise StreamlitMaxRetriesError(
+                    f"Unable to access file or folder: {path}"
+                ) from ex
+            # Continue with loop to either retry or raise MaxRetriesError.
 
-    raise MaxRetriesError(f"Unable to access file or folder: {path}")
+    raise StreamlitMaxRetriesError(f"Unable to access file or folder: {path}")
 
 
-def _retry_dance():
+def _retry_dance() -> Generator[int, None, None]:
     """Helper for writing a retry loop.
 
     This is useful to make sure all our retry loops work the same way. For example,
@@ -201,7 +205,3 @@ def _retry_dance():
     for i in range(_MAX_RETRIES):
         yield i
         time.sleep(_RETRY_WAIT_SECS)
-
-
-class MaxRetriesError(Error):
-    pass
